@@ -5,65 +5,7 @@ from dsmr_parser import telegram_specifications
 from dsmr_parser.clients import SerialReader, SERIAL_SETTINGS_V2_2
 from dsmr_parser import obis_references as obis_ref
 
-class DSMREntity(object):
-  """
-  Entity reading values from DSMR telegram.
-  """
-
-  def __init__(self, name, obis):
-    """Initialize entity."""
-    self._name = name
-    self._obis = obis
-    self.telegram = {}
-
-  def get_dsmr_object_attr(self, attribute):
-    """Read attribute from last received telegram for this DSMR object."""
-    # Make sure telegram contains an object for this entities obis
-    if self._obis not in self.telegram:
-        return None
-
-    # Get the attribute value if the object has it
-    dsmr_object = self.telegram[self._obis]
-    return getattr(dsmr_object, attribute, None)
-
-  @property
-  def name(self):
-    """Return the name of the sensor."""
-    return self._name
-
-  def state(self):
-    """Return the state of sensor, if available, translate if needed."""
-    from dsmr_parser import obis_references as obis
-
-    value = self.get_dsmr_object_attr('value')
-
-    if self._obis == obis.ELECTRICITY_ACTIVE_TARIFF:
-      return self.translate_tariff(value)
-
-    try:
-      value = round(float(value), 2)
-
-    except TypeError:
-      pass
-
-    return value
-
-  @property
-  def unit_of_measurement(self):
-    """Return the unit of measurement of this entity, if any."""
-    return self.get_dsmr_object_attr('unit')
-
-  @staticmethod
-  def translate_tariff(value):
-    """Convert 2/1 to normal/low."""
-    # DSMR V2.2: Note: Rate code 1 is used for low rate and rate code 2 is
-    # used for normal rate.
-    if value == '0002':
-      return 'normal'
-    elif value == '0001':
-      return 'low'
-    else:
-      return None
+from dsmr.telegram import Entity
 
 class DSMRReader():
 
@@ -215,53 +157,16 @@ class DSMRReader():
             ]
         ]
 
-        # self.devices = [DSMREntity(name, obis) for name, obis in self.obis_mapping]
-        self.devices = [DSMREntity(name, obis) for name, obis in self.obis_mapping_json]
-
-    def extract_data_from_telegram(self, telegram):
-      '''
-      Transform telegram to json
-      '''
-      # solar = self.read_solar()
-
-      # data = dict(
-      #   unix_timestamp = int(time.time()),
-      #   current_electricity_usage = str(telegram[obis_ref.CURRENT_ELECTRICITY_USAGE].value * 1000),
-      #   current_electricity_delivery = str(telegram[obis_ref.CURRENT_ELECTRICITY_DELIVERY].value * 1000),
-      #   electricity_active_tariff = str(telegram[obis_ref.ELECTRICITY_ACTIVE_TARIFF].value),
-      #   usage_total_low = str(telegram[obis_ref.ELECTRICITY_USED_TARIFF_1].value * 1000),
-      #   usage_total_high = str(telegram[obis_ref.ELECTRICITY_USED_TARIFF_2].value * 1000),
-      #   redelivery_total_low = str(telegram[obis_ref.ELECTRICITY_DELIVERED_TARIFF_1].value * 1000),
-      #   redelivery_total_high = str(telegram[obis_ref.ELECTRICITY_DELIVERED_TARIFF_2].value * 1000),
-      #   usage_gas_total = str(telegram[obis_ref.HOURLY_GAS_METER_READING].value * 1000)
-      # )
-
-      # TODO: add uom: telegram.CURRENT_ELECTRICITY_USAGE.unit
-
-      data = dict(
-        unix_timestamp = int(time.time()),
-        current_electricity_usage = str(telegram.CURRENT_ELECTRICITY_USAGE.value * 1000),
-        current_electricity_delivery = str(telegram.CURRENT_ELECTRICITY_DELIVERY.value * 1000),
-        electricity_active_tariff = str(telegram.ELECTRICITY_ACTIVE_TARIFF.value),
-        usage_total_low = str(telegram.ELECTRICITY_USED_TARIFF_1.value * 1000),
-        usage_total_high = str(telegram.ELECTRICITY_USED_TARIFF_2.value * 1000),
-        redelivery_total_low = str(telegram.ELECTRICITY_DELIVERED_TARIFF_1.value * 1000),
-        redelivery_total_high = str(telegram.ELECTRICITY_DELIVERED_TARIFF_2.value * 1000),
-        usage_gas_total = str(telegram.HOURLY_GAS_METER_READING.value * 1000)
-      )
-
-      if self.debug:
-          print(json.dumps(data, indent=2))
-
-      return data
+        # self.entities = [Entity(name, obis) for name, obis in self.obis_mapping]
+        self.entities = [Entity(name, obis) for name, obis in self.obis_mapping_json]
 
     def __update_entities_telegram(self, telegram):
         '''
         Update entities with latest telegram and trigger state update.
         '''
-        # Make all device entities aware of new telegram
-        for device in self.devices:
-            device.telegram = telegram
+        # Make all entities aware of new telegram
+        for entity in self.entities:
+            entity.telegram = telegram
         
 
     def read(self):
@@ -272,40 +177,29 @@ class DSMRReader():
           if self.debug:
             print('>> Reading DSMR in debug mode') 
 
-          t = dict()
+          for telegram in self.serial_reader.read():
 
-          for telegram in self.serial_reader.read_as_object():
-            # now, translate telegram to dict
-            obj = self.extract_data_from_telegram(telegram)
+            self.__update_entities_telegram(telegram)
 
-            # do something with 'obj'
-            
+            data = dict()
 
-          # for telegram in self.serial_reader.read():
+            # the combination of entities == dict/json
+            for entity in self.entities:
+                if entity.unit == 'kW':
+                    value = float(entity.state()) * 1000
+                else:
+                    value = entity.state()
 
-          #   if (self.debug):
-          #     print(telegram)
+                # print(f'{entity.name} == {value} {entity.unit or ""}')
 
-          #   self.__update_entities_telegram(telegram)
+                data[entity.name] = f'{value} {entity.unit or ""}'
 
-          #   data = dict()
-
-          #   for device in self.devices:
-          #       if device.unit_of_measurement == 'kW':
-          #           value = float(device.state()) * 1000
-          #       else:
-          #           value = device.state()
-
-          #       # print(f'{device.name} == {value} {device.unit_of_measurement or ""}')
-
-          #       data[device.name] = value
-
-          #   print(json.dumps(data, indent=2))
+            print(json.dumps(data, indent=2))
 
 
           #   print('=' * 80)
 
         except Exception as e:
-          print(str(e))
+          print(f'[ERROR] {str(e)}')
         
         
\ No newline at end of file
